%% Sloth's lambda lifting handling.
%%
%% Copyright (C) 2005-2006, Babel Group
%%
%% Written by: 
%%   * Emilio Jesús Gallego Arias
%%
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

:- module(lambda, [desugar_lambdas/1, new_lambda/2]).

:- use_module(symtab,     [add_rule/2] ).
:- use_module(curryast,   [split_rule/3, map_rules/2, map_tree/3,
	                   get_function/2 ]).
:- use_module(combinator, [make_combinator/3, replace_comb/3] ).

:- include(ast_ops).

%% Warning: This commentary is half-accurate. (In contrast with the
%% rest of the comments, that are completely inaccurate)

%% I don't know why Ciao doesn't reinstate asserted predicates when 
%% a search is in place, for more refence see util/bug3.pl, so we have 
%% to map the inner expression first.

desugar_lambdas(Module) :-
	map_rules(Module, new_lambda).

%% Simple lambda lifting.
new_lambda(lambda(Pattern, E), NE) :-
        map_tree(E, new_lambda, ME),
	%% Create the lambda function.
	split_rule(LRule, Pattern, ME),
	make_combinator(LRule, LCombinator, rinfo(LName, Args)),
	NE = LName@Args,
	globalize_ldef_func(LCombinator).

globalize_ldef_func(Func) :-
	split_rule(Func, Head, _),
	get_function(Head, FName),
	add_rule(FName, Func).

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

