%% Sloth's old lexer interface.
%%
%% Copyright (C) 2005-2006, Babel Group
%%
%% Written by: 
%%   * Emilio Jesús Gallego Arias
%%
%% Inspired by the old Sloth lexer.
%% 
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

:- module(lexer_compat, [init_scanner/1, init_scanner_string/1,
	                 token_have/1, token_look/1, token_back/0,
	                 token_skip/0, last_token/1]).
%
:- use_module(library(dynamic), [asserta/1, assertz/1, retract/1, retractall/1]).

:- use_module(lexer, [scanFile/2, scanTokensExpr/2, display_tokens/1] ).

:- dynamic tokens/1.
:- dynamic last_token_data/1.

% Compatibility module.

% Now init_scanner takes an argument, the string to 
% lex by now.

% It works lexing the string and storing it as a predicate
% that token_have will return step by step.

init_scanner(File) :-
	retractall(tokens(_)),
	retractall(last_token_data(_)),
	scanFile(File, Tokens),
	assert_tokens(Tokens),
	asserta(last_token_data((bof, (0,0)))).

init_scanner_string(String) :-
	retractall(tokens(_)),
	retractall(last_token_data(_)),
	scanTokensExpr(String, Tokens),
	assert_tokens(Tokens),
	asserta(last_token_data((bof, (0,0)))).

%% The old method (storing the full list of tokens in tokens) was
%% terribly slow.
assert_tokens([]) :- !.
assert_tokens([T|TL]) :-
	assertz(tokens(T)),
	!,
	assert_tokens(TL).
	
%  token_have(Token)
%
%  token_have(Token), gets a token from tokens and updates
%  it.

token_have(Token) :-
	(
	    tokens((PToken, _Pos)),
	    !,
	    Token = PToken,
	    retract(tokens(T)),
	    asserta(last_token_data(T)),
	    !
	;
	    tokens(T),
	    throw(error(lexer_compat, wtf_non_standar_token(T)))
	).

%  token_look(Token)
%
%  token_look/1 Token is the next token in the input (used with next).

token_look(Token) :-
	tokens((PToken, _Pos)),
	!,
	Token = PToken.

last_token(Token) :-
	last_token_data(Token),
	!.

% skip(Token)
%
% skip/1  skip the next token in the input.

token_skip :-
	!,
	retract(tokens(T)),
	asserta(last_token_data(T)),
	!.

token_back :-
	!,
	retract(last_token_data(T)),
	asserta(tokens(T)),
	!.

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

