%% Sloth's module handling predicates
%% Copyright (C) 2005-2006, Babel Group
%%
%% Written by: 
%%   * Emilio Jesús Gallego Arias
%%
%% This program is free software; you can redistribute it and/or
%% modify it under the terms of the GNU General Public License
%% as published by the Free Software Foundation; either version 2
%% of the License, or (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

:- module(modules, [load_module/3, unload_module/1, 
	            module_is_loaded/1, get_loaded_modules/1]).
%%
:- use_module(library(dynamic),    [assert/1, retract/1] ).
:- use_module(library(aggregates), [findall/3] ).
:- use_module(library(compiler),   [unload/1]).

%% 
:- use_module(exports, [load_exports/2]).
:- use_module(symtab,  [unload_interfaces/1]).

:- use_module(scope,             [close_named_scope/1] ).
:- use_module(cmdopts,           [cmd_options/2]).

%% This module implements some predicates for the module system.
:- data current_modules/1.

module_is_loaded(Module) :-
	current_modules(Module).

get_loaded_modules(ModList) :-
	findall(M, current_modules(M), ModList).

%% A module can be loaded by specifying its name and allowed imports.
%% An engine module has no interface.
load_module(Name, _Imps, engine) :-
	!,
	use_module(sloth(Name)),
	assert(current_modules(Name)).
	
load_module(Name, Imports, _Flags) :-
	(
	    current_modules(Name) ->
	    unload_module(Name),
	    load_module(Name, Imports, _Flags)
	;
	    %% use_module(sloth(Name)),
	    load_exports(Name, Imports),
	    !,
	    assert(current_modules(Name))
	).

unload_module(Module) :-
	(
	    unload(Module) % This kills tohnf, etc...
        ;
	    true
	),
	unload_interfaces(Module),
	(
	    close_named_scope(imported(Module))
	;
	    true
	),
	retract(current_modules(Module)),
	!.


%% Search order.
:- multifile file_search_path/2.
:- dynamic file_search_path/2.

file_search_path(sloth, LibDir) :-
	cmd_options('-l', [LibDir]).
file_search_path(sloth, '.').

%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:
