:- module(priotree, [pt_insert/3]).

%% A simple priority tree without rebalancing.
%% Useful for parsing grammars with variable ops.

:- include(ast_ops).

:- use_module(symtab, [get_object_property/3]).

insert(Op@[E1, E2], (NOp, NE), NOp@[Op@[E1, E2], NE]) :-
	priority(Op,  P1),
	priority(NOp, P2),
	P1 > P2,
	!.

pt_insert(Op@[E1, E2], (NOp, NE), NTree) :-
	priority(Op, P1),
	priority(NOp, P2),
	P1 = P2,
	(
	    assoc(Op, infix)
	;
	    assoc(Op, infixl)
	),
	!,
	NTree = Op@[E1, NOp@[E2, NE]].
	
pt_insert(Op@[E1, E2], (NOp, NE), NTree) :-
	priority(Op,  P1),
	priority(NOp, P2),
	P1 = P2,
	assoc(Op, infixr),
	!,
	NTree = NOp@[Op@[E1, E2], NE].

pt_insert(Op@[E1, E2], (NOp, NE), NTree) :-
	priority(Op,  P1),
	priority(NOp, P2),
	P1 < P2,
	!,
	pt_insert(E2, (NOp, NE), NTree1),
	NTree = Op@[E1, NTree1].

pt_insert(E1, (NOp, E2), NOp@[E1, E2]) :- !.

%% FIXME, ticket #24
priority(qid('Prelude',:), 9).
priority(Op, P) :-
	get_object_property(Op, priority, P).

assoc(qid('Prelude',:), infixl).
assoc(Op, Assoc) :-
	get_object_property(Op, assoc, Assoc).


%% Local Variables: 
%% mode: CIAO
%% update-version-comments: "off"
%% End:

